/**
* @file  file_display_colors.cpp
* @brief Implementierung der Klasse FileDisplayColors.
*/
//////////////////////////////////////////////////////////////////////

#include "file/file_display_colors.h"
#include <memory.h>
#include <stdio.h>

FileDisplayColors::FileDisplayColors()
{
    // Initialisieren
    resetData();

    // Fehlerobjekt zuruecksetzen
    resetError();

    // Der Standardmäßig zu verarbeitende Datensatztyp
    setRecordType( Record_type::display_color );
}

FileDisplayColors::~FileDisplayColors()
{
    // Speicher freigeben
    resetData();
}

void FileDisplayColors::resetData()
{
    // Basis Reset
    FileIntelHex::resetData();

    // Member initialisieren
    m_pDisplayColorsNode = (DISPLAYCOLORNODE*)nullptr;
}

void FileDisplayColors::Init( unsigned char count )
{
    int i;
    int j;

    // Variable fuer Groessenangabe
    m_buffer.resize( sizeof(DISPLAYCOLORNODE) + count * sizeof(unsigned int) + count * sizeof(DISPLAYCOLOR) );

    m_pDisplayColorsNode = reinterpret_cast<DISPLAYCOLORNODE *>( &m_buffer[0] );

    // 1.
    // Zeichensaetze init
    // Grundstruktur aufsetzen
    m_pDisplayColorsNode->version = 1;
    m_pDisplayColorsNode->cnt     = count;
    m_pDisplayColorsNode->temp[0] = 0;
    m_pDisplayColorsNode->temp[1] = 0;
    m_pDisplayColorsNode->ptrColorArray = sizeof(DISPLAYCOLORNODE);

    // Zeigerarray und DisplayColorsstrukturzeiger initialisieren
    i = m_pDisplayColorsNode->cnt;
    for (j=0; j < i; j++) {

        // Zeigereintrag auf DisplayColorstruktur
        *(unsigned int*)&m_buffer[m_pDisplayColorsNode->ptrColorArray + j * sizeof(unsigned int)] = 
            m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR);

        m_pDisplayColors[j] = (DISPLAYCOLOR*)&m_buffer[m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR)];
    }

}

int FileDisplayColors::SetDisplayColor(int idx, DISPLAYCOLOR dc)
{
    if ( idx >= m_pDisplayColorsNode->cnt )
        return -1;

    *m_pDisplayColors[idx] = dc;
    return idx;
}

DISPLAYCOLOR FileDisplayColors::GetDisplayColor(int idx)
{
    DISPLAYCOLOR dc;
    memset( &dc, 0, sizeof(DISPLAYCOLOR) );

    if ( idx >= m_pDisplayColorsNode->cnt )
        return dc;

    return *m_pDisplayColors[idx];
}

bool FileDisplayColors::load( const std::string &filePath )
{
    // Lesen der Hexdatei
    if ( FileIntelHex::load( filePath ) )
    {
        // Member durch gelesene Daten initialisieren und pruefen
        if ( m_buffer.size() )
        {
            int i;
            int j;

            m_pDisplayColorsNode = reinterpret_cast<DISPLAYCOLORNODE *>( &m_buffer[0] );
            // 1.
            // DisplayColors pruefen

            // Zeigerarray und DisplayColorstrukturzeiger initialisieren
            i = m_pDisplayColorsNode->cnt;
            if ( i > MAX_DISPLAYCOLORS )
            {
                // Fehler zuweisen
                setErrorState( Error_state::format );
                setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
            }
            else
            {
                for ( j = 0; j < i; j++ )
                    m_pDisplayColors[j] = (DISPLAYCOLOR*)&m_buffer[m_pDisplayColorsNode->ptrColorArray + i * sizeof(unsigned int) + j * sizeof(DISPLAYCOLOR)];
            }

        }
        else
        {
            // Fehler zuweisen
            setErrorState( Error_state::no_data );
            setErrorDetail( boost::str( boost::format( "%1%(%2%): %3%" ) % __FILE__ % __LINE__ % errorText() ) );
        }
    }

    return (errorState() == Error_state::none);
}

bool FileDisplayColors::save( const std::string &filePath, bool appendToFile /*= false*/, bool appendEndLine /*= true */ ) 
{
    return FileIntelHex::save( filePath, appendToFile, appendEndLine );
}
