#include "file/interface/device_update_classes.h"
#include <algorithm>
#include <boost/format.hpp>
#include <boost/algorithm/string/replace.hpp>

namespace Datafox
{

//#####################################

DeviceUpdateSegmentFirmware::DeviceUpdateSegmentFirmware()
{
    // Struktur-Id lokal halten.
    _id = StructIdDeviceUpdateSegmentFirmware;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    _data = &data();
#endif
}

DeviceUpdateSegmentFirmware::DeviceUpdateSegmentFirmware( const DeviceUpdateSegmentFirmware &segment )
{
    // Struktur-Id lokal halten.
    _id = StructIdDeviceUpdateSegmentFirmware;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    _data = &data();
#endif

    // Header des Segmentes kopieren.
    std::copy_n( segment._structData.begin(), segment._structData.size(), _structData.begin() );
    // Daten des Segmentes kopieren.
    segmentData_ = segment.segmentData_;
}

void DeviceUpdateSegmentFirmware::resetData()
{
    // Daten löschen.
    std::fill( _structData.begin(), _structData.end(), 0 );

    // Segmentdaten loeschen
    segmentData_.clear();

    // Headerdaten eintragen
    data().header.id = id();
    data().header.size = size();
}

EnumStructId DeviceUpdateSegmentFirmware::id() const
{
    return _id;
}

int DeviceUpdateSegmentFirmware::size() const
{
    return sizeof(StructDeviceUpdateSegmentFirmware);
}

StructDeviceUpdateSegmentFirmware &DeviceUpdateSegmentFirmware::data()
{
    return reinterpret_cast<StructDeviceUpdateSegmentFirmware &>( _structData );
}

DeviceUpdateSegmentFirmware::SerializeType::size_type DeviceUpdateSegmentFirmware::sizeSegmentData() const
{
    return segmentData_.size();
}

DeviceUpdateSegmentFirmware::SerializeType &DeviceUpdateSegmentFirmware::segmentData()
{
    return segmentData_;
}

void DeviceUpdateSegmentFirmware::writeSerialize( SerializeType::iterator iter )
{
    // Headerdaten als Kopie übernehmen.
    std::copy_n( _structData.begin(), _structData.size(), iter );
    // Daten als Kopie übernehmen.
    iter += _structData.size();
    std::copy_n( segmentData_.begin(), segmentData_.size(), iter );
}

void DeviceUpdateSegmentFirmware::writeSerialize( SerializeType &buffer )
{
    // Anfangsposition halten.
    auto offsBegin = buffer.size();

    // Groesse fuer hinzufuegung anpassen.
    buffer.resize( buffer.size() + size() + sizeSegmentData() );

    // Iterator auf den Beginn halten.
    auto iterBegin = buffer.begin() + offsBegin;

    // CRC !!!!!!!

    // Die Headerstruktur eintragen.
    writeSerialize( iterBegin );
}

void DeviceUpdateSegmentFirmware::readSerialize( SerializeType::const_iterator iter )
{
    // Headerdaten als Kopie übernehmen.
    std::copy_n( iter, _structData.size(), _structData.begin() );
    // Daten als Kopie übernehmen.
    iter += _structData.size();
    segmentData_.resize( data().size );
    std::copy_n( iter, segmentData_.size(), segmentData_.begin() );
}

//#####################################

DeviceUpdateSegment::DeviceUpdateSegment()
{
    // Struktur-Id lokal halten.
    _id = StructIdDeviceUpdateSegment;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    _data = &data();
#endif
}

DeviceUpdateSegment::DeviceUpdateSegment( const DeviceUpdateSegment &segment )
{
    // Struktur-Id lokal halten.
    _id = StructIdDeviceUpdateSegment;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    _data = &data();
#endif

    // Header des Segmentes kopieren.
    std::copy_n( segment._structData.begin(), segment._structData.size(), _structData.begin() );
    // Daten des Segmentes kopieren.
    segmentData_ = segment.segmentData_;
}

void DeviceUpdateSegment::resetData()
{
    // Headerdaten löschen.
    std::fill( _structData.begin(), _structData.end(), 0 );

    // Daten löschen.
    segmentData_.clear();

    // Headerdaten eintragen
    data().header.id = id();
    data().header.size = size();
}

EnumStructId DeviceUpdateSegment::id() const
{
    return _id;
}

int DeviceUpdateSegment::size() const
{
    return sizeof(StructDeviceUpdateSegmentFirmware);
}

StructDeviceUpdateSegment &DeviceUpdateSegment::data()
{
    return reinterpret_cast<StructDeviceUpdateSegment &>( _structData );
}

DeviceUpdateSegment::SerializeType::size_type DeviceUpdateSegment::sizeSegmentData() const
{
    return segmentData_.size();
}

DeviceUpdateSegment::SerializeType &DeviceUpdateSegment::segmentData()
{
    return segmentData_;
}

void DeviceUpdateSegment::writeSerialize( SerializeType::iterator iter )
{
    // Headerdaten als Kopie übernehmen.
    std::copy_n( _structData.begin(), _structData.size(), iter );
    // Daten als Kopie übernehmen.
    iter += _structData.size();
    std::copy_n( segmentData_.begin(), segmentData_.size(), iter );
}

void DeviceUpdateSegment::writeSerialize( SerializeType &buffer )
{
    // Anfangsposition halten.
    auto offsBegin = buffer.size();

    // Groesse fuer hinzufuegung anpassen.
    buffer.resize( buffer.size() + size() + sizeSegmentData() );

    // Iterator auf den Beginn halten.
    auto iterBegin = buffer.begin() + offsBegin;

    // CRC !!!!!!!

    // Die Headerstruktur eintragen.
    writeSerialize( iterBegin );   
}

void DeviceUpdateSegment::readSerialize( SerializeType::const_iterator iter )
{
    // Headerdaten als Kopie übernehmen.
    std::copy_n( iter, _structData.size(), _structData.begin() );
    // Daten als Kopie übernehmen.
    iter += _structData.size();
    segmentData_.resize( data().size );
    std::copy_n( iter, segmentData_.size(), segmentData_.begin() );
}

// ####################################

DeviceUpdateNode::DeviceUpdateNode()
{
    // Struktur-Id lokal halten.
    _id = StructIdDeviceUpdateNode;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    _data = &data();
#endif
}

void DeviceUpdateNode::resetData()
{
    // Daten löschen.
    std::fill( _structData.begin(), _structData.end(), 0 );

    // Headerdaten eintragen
    data().header.id = id();
    data().header.size = static_cast<unsigned short>( size() );

    // Segment der Firmwaredatei.
    _segmentFirmware.resetData();

    // Klasse für Arrays
    _array.resetData();

    // Alle Segmente löschen
    _segments.clear();
}

EnumStructId DeviceUpdateNode::id() const
{
    return _id;
}

std::size_t DeviceUpdateNode::size() const
{
    return sizeof(StructDeviceUpdateNode);
}

StructDeviceUpdateNode &DeviceUpdateNode::data()
{
    return reinterpret_cast<StructDeviceUpdateNode &>( _structData );
}

void DeviceUpdateNode::writeSerialize( SerializeType::iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( _structData.begin(), _structData.size(), iter );
}

bool DeviceUpdateNode::writeSerialize( SerializeType &buffer )
{
    // Ggf. vorhandenen Pufferspeicher löschen.
    if ( buffer.size() )
        buffer.clear();

    // Fuer Hauptstruktur speicher allokieren.
    buffer.resize( sizeof(StructDeviceUpdateNode) );

    // Iterator auf den Beginn halten.
    auto iterBegin = buffer.begin();
    auto offsBegin = std::distance( buffer.begin(), iterBegin );

    // Offset auf das Segment der Firmware eintragen.
    data().offsetSegmentFirmware = static_cast<unsigned int>( std::distance( iterBegin, buffer.end() ) );

    // Schreiben der Daten zum Segment.
    _segmentFirmware.writeSerialize( buffer );

    // Ggf. durch resize inerhalb writeSerialize ungültig geworden.
    iterBegin = buffer.begin();

    // Sind Angaben zu DeviceUpdateSegment vorhanden?
    if ( _segments.size() )
    {
        // Offset auf StructArray für die einzelnen Strukturen eintragen.
        data().offsetSegmentArray = static_cast<unsigned int>( std::distance( iterBegin, buffer.end() ) );

        // Größe für dieses Array anpassen.
        buffer.resize( buffer.size() + _array.size() + _segments.size() * sizeof(unsigned int) );
        // Neu zuweisen.
        iterBegin = buffer.begin() + offsBegin;

        // Eintragen des Array und der Strukturen
        auto iterArray = iterBegin + data().offsetSegmentArray;
        // Anzahl der Strukturen eintragen.
        _array.data().count = static_cast<unsigned short>( _segments.size() );
        // Schreiben der Array-Struktur
        _array.writeSerialize( iterArray );
        // Auf Beginn OffsetArray
        iterArray += _array.size();

        // Angaben
        int i;
        auto iter = _segments.begin();
        for ( i = 0; iter != _segments.end(); ++iter, ++i )
        {
            auto offsBuffer = std::distance( buffer.begin(), iterArray );
            unsigned int &offs = reinterpret_cast<unsigned int &>( *(iterArray + i * sizeof(unsigned int)) );
            offs = static_cast<unsigned int>( std::distance( iterArray, buffer.end() ) );

            (*iter).writeSerialize( buffer );

            // Ggf. durch resize inerhalb writeSerialize ungültig geworden.
            iterArray = buffer.begin() + offsBuffer;
        }
    }
    else
    {
        // Offset auf StructArray ist leer.
        data().offsetSegmentArray = 0;
    }    

    // Ggf. durch resize inerhalb writeSerialize ungültig geworden.
    iterBegin = buffer.begin();

    // DeviceUpdateNode eintragen.
    writeSerialize( iterBegin );

    return true;
}

void DeviceUpdateNode::readSerialize( SerializeType::const_iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( iter, _structData.size(), _structData.begin() );
}

/*! Serialisiert das ByteArray in eine Baumstruktur von HardwareConfigs.
    Aus dem zugrunde gelegten ByteArray wird eine Baumartige Speicherrepresentation
    von HardwareConfigs erzeugt. Über diese kann dann auf einfache
    Art mit den Daten gearbeitet werden.
*/
bool DeviceUpdateNode::readSerialize( const SerializeType &buffer )
{
    // Hauptstruktur lesen.
    readSerialize( buffer.begin() );

    // Iterator auf den Begin halten.
    auto iterBegin = buffer.begin();

    // Segment zur Firmware lesen.
    _segmentFirmware.readSerialize( iterBegin + data().offsetSegmentFirmware );

    // Sind Angaben vorhanden?
    if ( data().offsetSegmentArray )
    {
        // Datenarray lesen 
        auto iterArray = iterBegin + data().offsetSegmentArray;
        _array.readSerialize( iterArray );

        // Auf den Beginn des Arrays wechseln.
        iterArray += _array.size();

        // Anzahl anpassen.
        _segments.resize( _array.data().count );

        int i;
        for ( i = 0; i < _array.data().count; ++i )
        {
            // Offset auf die Daten des Eintrags lesen.
            const unsigned int &offs = reinterpret_cast<const unsigned int &>( *(iterArray + i * sizeof(unsigned int)) );

            _segments[i].readSerialize( iterArray + offs );
        }
    }

    return true;
}

void DeviceUpdateNode::setSegmentFirmware( const StructDeviceUpdateSegmentFirmware &dusf, const SerializeType &segmentData )
{
    // Header des Segmentes.
    _segmentFirmware.data() = dusf;
    // Daten des Segmentes.
    _segmentFirmware.segmentData() = segmentData;
}

const StructDeviceUpdateSegmentFirmware &DeviceUpdateNode::getSegmentFirmware()
{
    return _segmentFirmware.data();
}

const DeviceUpdateNode::SerializeType &DeviceUpdateNode::getSegmentDataFirmware()
{
    return _segmentFirmware.segmentData();
}

std::size_t DeviceUpdateNode::countSegments()
{
    return _segments.size();
}

void DeviceUpdateNode::appendSegment( const StructDeviceUpdateSegment &segment, const SerializeType &segmentData  )
{
    DeviceUpdateSegment item;
    // Header des Segmentes.
    item.data() = segment;
    // Daten des Segmentes
    item.segmentData() = segmentData;

    _segments.push_back( item );
}

const DeviceUpdateSegment &DeviceUpdateNode::getSegment( int index )
{
    return reinterpret_cast<const DeviceUpdateSegment &>( _segments[index] );
}

} // namespace Datafox
