#include "file/interface/exchange_classes.h"
#include <boost/format.hpp>

namespace Datafox
{

FileHeader::FileHeader( EnumFileId id )
{
    // Struktur-Id lokal halten.
    m_id = id;

    // Initalisieren
    resetData();
}

void FileHeader::resetData()
{
    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );

    // Headerdaten eintragen
    data().df[0] = 'D';
    data().df[1] = 'F';
    data().id    = id();
}

EnumFileId FileHeader::id() const
{
    return m_id;
}

int FileHeader::size() const
{
    return sizeof(StructFileHeader);
}

void FileHeader::writeSerialize( SerializeType::iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( m_structData.begin(), m_structData.size(), iter );
}

void FileHeader::readSerialize( SerializeType::const_iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( iter, m_structData.size(), m_structData.begin() );
}

void FileHeader::readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer )
{
    (void)buffer;
    readSerialize( iter );
}

// ########################################################

StructArray::StructArray()
{
    // Struktur-Id lokal halten.
    m_id = StructIdArray;

    // Initalisieren
    resetData();

#ifdef _DEBUG
    m_data = &data();
#endif
}

void StructArray::resetData()
{
    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );

    // Headerdaten eintragen
    data().header.id = id();
    data().header.size = size();
}

EnumStructId StructArray::id() const
{
    return m_id;
}

int StructArray::size() const
{
    return sizeof( StructStructArray );
}

StructStructArray &StructArray::data()
{
    return reinterpret_cast<StructStructArray &>( m_structData );
}

void StructArray::writeSerialize( SerializeType::iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( m_structData.begin(), m_structData.size(), iter );
}

void StructArray::readSerialize( SerializeType::const_iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( iter, m_structData.size(), m_structData.begin() );
}

void StructArray::readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer )
{
    (void)buffer;
    readSerialize( iter );
}

std::string StructArray::toJson()
{
    std::string text;
    text = "{";
    text += "\"header\": {";
    text += boost::str( boost::format("\"id\": %d,") % static_cast<int>( data().header.id ) );
    text += boost::str( boost::format("\"size\": %d") % static_cast<int>( data().header.size ) );
    text += "},";
    text += boost::str( boost::format("\"count\": %d") % static_cast<int>( data().count ) );
    text += "}";

    return text;
}

/*
template<class StructExchange> ExchangeBase<StructExchange>::ExchangeBase( StructId id )
{
    // Struktur-Id lokal halten.
    m_id = id;

    // Initalisieren
    reset();
}

template<class StructExchange> void ExchangeBase<StructExchange>::reset()
{
    // Ggf. vorhandene Daten löschen.
    m_structData.clear();

    // Für die Daten Speicher reservieren.
    m_structData.resize( size() );

    // Daten löschen.
    std::fill( m_structData.begin(), m_structData.end(), 0 );

    // Headerdaten eintragen
    data().header.id = id();
    data().header.size = size();
}

template<class StructExchange> const StructId ExchangeBase<StructExchange>::id() const
{
    return m_id;
}

template<class StructExchange> const int ExchangeBase<StructExchange>::size() const 
{
    return sizeof( StructExchange );
}

template<class StructExchange> StructExchange &ExchangeBase<StructExchange>::data()
{
    return reinterpret_cast<StructExchange &>( m_structData );
}

template<class StructExchange> void ExchangeBase<StructExchange>::writeSerialize( SerializeType::iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( m_structData.begin(), m_structData.size(), iter );
}

template<class StructExchange> void ExchangeBase<StructExchange>::readSerialize( SerializeType::const_iterator iter )
{
    // Daten als Kopie übernehmen.
    std::copy_n( iter, m_structData.size(), m_structData.begin() );
}

template<class StructExchange> void ExchangeBase<StructExchange>::readSerialize( SerializeType::const_iterator iter, const SerializeType &buffer )
{
    readSerialize( iter );
}
*/
} // namespace Datafox
